untyped
global function rainPrecache
global function rainSettings
const asset FX_RAIN_PARTICLE = $"P_weather_rain_ts"
const asset FX_LIGHTNING_PARTICLE = $"P_lightning_4096"

void function rainSettings(){
    #if UI
        #if S2_RAIN_HAS_MOD_SETTINGS
            AddConVarSettingEnum("s2_rain_switch", "Rain On/Off", "S2.RainAndLightning", ["Off", "On"])
            AddConVarSettingEnum("s2_lightning_switch", "Lightning On/Off", "S2.RainAndLightning", ["Off", "On"])
            AddConVarSetting("s2_lightning_number", "Lightning Number", "S2.RainAndLightning", "int")
            AddConVarSetting("s2_lightning_frequency", "Lightning Frequency", "S2.RainAndLightning", "int")
            AddConVarSetting("s2_lightning_frequency_deviation", "Lightning Frequency Deviation", "S2.RainAndLightning", "float")
            AddConVarSetting("s2_lightning_radius", "Lightning Radius", "S2.RainAndLightning", "int")
            AddConVarSetting("s2_lightning_radius_deviation", "Lightning Radius Deviation", "S2.RainAndLightning", "float")
        #else
            thread noSettingsDiagThread()
        #endif
    #endif
}

#if UI

void function noSettingsDiagThread(){
    WaitFrame()
    noSettingsDiag()
}

void function noSettingsDiag(){
    DialogData errorPopUp
    errorPopUp.header = "MISSING REQUIRED DEPENDENCY"
    errorPopUp.message = "RainAndLightning requires ModSettings to function.\nThe mod was not found; please download it."
    errorPopUp.image = $"ui/menu/common/dialog_error"
    AddDialogButton(errorPopUp, "Exit", void function(){ ClientCommand("exit") })
    AddDialogButton(errorPopUp, "Download", void function(){
        LaunchExternalWebBrowser("https://northstar.thunderstore.io/package/EladNLG/ModSettings/", WEBBROWSER_FLAG_FORCEEXTERNAL)
        noSettingsDiag()
    })
    OpenDialog(errorPopUp)
}
#endif


void function rainPrecache(){
    #if CLIENT
    AddCallback_EntitiesDidLoad(startRain)
    #endif
}

#if CLIENT

void function startRain(){
    WaitFrame()
    if(IsLobby())
        return
    level.g_weatherFXHandle <- null
    PrecacheParticleSystem(FX_RAIN_PARTICLE)
    PrecacheParticleSystem(FX_LIGHTNING_PARTICLE)
    entity player = GetLocalClientPlayer()
    string currentMapName = GetMapName()
    thread mapLighting(currentMapName, player)
    while(GetMapName() == currentMapName){
        if(GetConVarInt("s2_rain_switch") == 1)
            playerRain(player)
        else{
            if(level.g_weatherFXHandle != null && EffectDoesExist(level.g_weatherFXHandle))
                EffectStop(level.g_weatherFXHandle, true, false)
        }
        wait 0.5
    }
}

void function mapLighting(string currentMapName, entity player){
    while(GetMapName() == currentMapName){
        Wait(GetConVarFloat("s2_lightning_frequency"))
        if(!IsValid(player) || !IsAlive(player) || IsWatchingReplay() || player == null)
		    continue
        if(GetConVarInt("s2_lightning_switch") == 1){
            for(int count = GetConVarInt("s2_lightning_number"); count > 0; count--)
                thread singularLightning(player.GetOrigin())
        }
    }
}

void function singularLightning(vector playerPos){
    Wait(RandomFloatRange(0.0, (2* GetConVarFloat("s2_lightning_frequency_deviation") * GetConVarFloat("s2_lightning_frequency"))))
    float randomDeg = RandomFloatRange(0.0, (2*PI))
    float useRadius = GetConVarFloat("s2_lightning_radius") * (RandomFloatRange(-GetConVarFloat("s2_lightning_radius_deviation"), GetConVarFloat("s2_lightning_radius_deviation")) + 1.0)
    vector LightingPos = playerPos + Vector(useRadius * cos(randomDeg), useRadius * sin(randomDeg), 600.0)
    StartParticleEffectInWorld(GetParticleSystemIndex(FX_LIGHTNING_PARTICLE), LightingPos, <0,0,0>)
}

void function playerRain(entity player){
    if(!IsValid(player) || !IsAlive(player) || IsWatchingReplay() || player == null)
		return
    if(level.g_weatherFXHandle != null && EffectDoesExist(level.g_weatherFXHandle))
        return
	level.g_weatherFXHandle = StartParticleEffectOnEntityWithPos(player, GetParticleSystemIndex(FX_RAIN_PARTICLE), FX_PATTACH_ABSORIGIN_FOLLOW, -1, <0, 0, 800>, <0, 0, 0>)
}

#endif